@@ -1,36 +1,39 @@
 # ABSTRACT:RESTful Web Services with login, persistent data, multiple in/out formats, IP security, role based access
-# Multiple input/output formats : JSON , XML , YAML, PERL , HUMAN
+# Multiple input/output formats : json , xml , yaml, perl , human
 #
 # George Bouras , george.mpouras@yandex.com
 # Joan Ntzougani, ✞
 
 package Dancer2::Plugin::WebService;
-our	$VERSION = '4.3.7';
+our	$VERSION = '4.3.8';
 use	strict;
 use	warnings;
 use Encode;
 use	Dancer2::Plugin;
 use	Storable;
-use	XML::Hash::XS; $XML::Hash::XS::utf8=1;  $XML::Hash::XS::encoding='utf-8';$XML::Hash::XS::root='root'; $XML::Hash::XS::canonical=0; $XML::Hash::XS::indent=2; $XML::Hash::XS::xml_decl=0; $XML::Hash::XS::doc=0;
-use	Data::Dumper;  $Data::Dumper::Terse=1;  $Data::Dumper::Trailingcomma=0;  $Data::Dumper::Deepcopy=1; $Data::Dumper::Indent=2; $Data::Dumper::Purity=1; $Data::Dumper::Sortkeys=0;
-use YAML::Syck;    $YAML::Syck::Headless=0; $YAML::Syck::ImplicitTyping=1;   $YAML::Syck::ImplicitUnicode=0;
+use	XML::Hash::XS; $XML::Hash::XS::canonical=0;    $XML::Hash::XS::utf8=1;  $XML::Hash::XS::doc=0;  $XML::Hash::XS::root='root'; $XML::Hash::XS::encoding='utf-8'; $XML::Hash::XS::indent=2; $XML::Hash::XS::xml_decl=0;
+use	Data::Dumper;  $Data::Dumper::Trailingcomma=0; $Data::Dumper::Indent=2; $Data::Dumper::Terse=1; $Data::Dumper::Deepcopy=1; $Data::Dumper::Purity=1; $Data::Dumper::Sortkeys=0;
+use YAML::Syck;    $YAML::Syck::ImplicitTyping=1;  $YAML::Syck::Headless=0; $YAML::Syck::ImplicitUnicode=0;
 use Cpanel::JSON::XS; my $JSON=Cpanel::JSON::XS->new; $JSON->space_before(1);$JSON->canonical(0);  $JSON->allow_tags(1);  $JSON->allow_unknown(0); $JSON->pretty(0); $JSON->indent(1); $JSON->space_after(1); $JSON->max_size(0); $JSON->relaxed(0); $JSON->shrink(0); $JSON->allow_nonref(0); $JSON->allow_blessed(1); $JSON->convert_blessed(1); $JSON->max_depth(1024); $JSON->utf8(0);
 
+
 if ($^O=~/(?i)MSWin/) {warn "Operating system is not supported\n"; exit 1}
+my %TokenDB = ();
+my %Formats = ( json=>'application/json', xml=>'text/xml', yaml=>'text/x-yaml', perl=>'text/html', human=>'text/html' );
+my $fmt_rgx; $_ = join '|', sort keys %Formats; $fmt_rgx = qr/^($_)$/;
+
 
 has error           => (is=>'rw', lazy=>1, default=> 0);
 has sort            => (is=>'rw', lazy=>1, default=> 0);
 has pretty          => (is=>'rw', lazy=>1, default=> 1);
-has formats         => (is=>'ro', lazy=>0, default=> sub{ {json=> 'application/json', xml=> 'text/xml', yaml=> 'text/x-yaml', perl=> 'text/html', human=> 'text/html'} });
-has formats_regex   => (is=>'ro', lazy=>0, default=> sub{ $_ = join '|', sort keys %{ $_[0]->formats }; $_ = qr/^($_)$/; $_ });
-has Format          => (is=>'rw', lazy=>1, default=> sub{ {from => undef, to => undef} });
 has route_name      => (is=>'rw', lazy=>1, default=> '');
 has ClientIP        => (is=>'rw', lazy=>1, default=> '');
 has reply_text      => (is=>'rw', lazy=>1, default=> '');
 has auth_method     => (is=>'rw', lazy=>1, default=> '');
 has auth_command    => (is=>'rw', lazy=>1, default=> '');
 has auth_config     => (is=>'rw', lazy=>1, default=> sub{ {} });
-has data            => (is=>'rw', lazy=>1, default=> sub{ {} });	# user posted data as hash
+has data            => (is=>'rw', lazy=>1, default=> sub{ {} }); # user posted data as hash
+has Format          => (is=>'rw', lazy=>1, default=> sub{ {from => undef, to => undef} });
 has Session_timeout => (is=>'ro', lazy=>0, from_config=>'Session idle timeout',default=> sub{ 3600 }, isa => sub {unless ( $_[0]=~/^\d+$/ ) {warn "Session idle timeout \"$_[0]\" It is not a number\n"; exit 1}} );
 has rules           => (is=>'ro', lazy=>0, from_config=>'Allowed hosts',       default=> sub{ ['127.0.*', '192.168.*', '172.16.*'] });
 has rules_compiled  => (is=>'ro', lazy=>0, default=> sub {my $array = [@{$_[0]->rules}]; for (@{$array}) { s/([^?*]+)/\Q$1\E/g; s|\?|.|g; s|\*+|.*?|g; $_ = qr/^$_$/i } $array});
@@ -48,8 +51,6 @@ WALKER => sub { my $data = shift; $Handler{ref $data}->($data, shift, \@_) }
 );
 
 
-my %TokenDB;
-
 
 sub BUILD
 {
@@ -65,8 +66,8 @@ $plg->config->{Routes}->{logout}     = {Protected=>1, Groups=>[]};
 $app->config->{charset}        //= 'UTF-8';
 $app->config->{encoding}       //= 'UTF-8';
 $app->config->{show_errors}    //= 0;
-$plg->config->{'Default format'} = 'json' if ((! exists $plg->config->{'Default format'}) || ($plg->config->{'Default format'} !~ $plg->formats_regex));
-$app->config->{content_type}     = $plg->formats->{ $plg->config->{'Default format'} };
+$plg->config->{'Default format'} = 'json' if ((! exists $plg->config->{'Default format'}) || ($plg->config->{'Default format'} !~ $fmt_rgx));
+$app->config->{content_type}     = $Formats{ $plg->config->{'Default format'} };
 
 # Module directory
 (my $module_dir =__FILE__) =~s|/[^/]+$||;
@@ -87,7 +88,7 @@ unless (-d $module_dir) {warn "Could not find the Dancer2::Plugin::WebService in
 
 			if ((exists $method->{'Use sudo'}) && ($method->{'Use sudo'}=~/(?i)[y1t]/)) {
 			my $sudo = undef;
-			foreach ('/bin', '/usr/bin', '/usr/sbin', '/sbin') { if ((-f "$_/sudo") && -x ("$_/sudo")) { $sudo="$_/sudo"; last } }
+			foreach (qw[/usr/bin /bin /usr/sbin /sbin]) { if ((-f "$_/sudo") && -x ("$_/sudo")) { $sudo="$_/sudo"; last } }
 			unless (defined $sudo) {warn "Could not found sudo command\n"; exit 1}
 			$plg->auth_command( "$sudo \Q$method->{Command}\E" )
 			}
@@ -128,19 +129,18 @@ delete $plg->config->{'Authentication methods'};
 		}
 	}
 
-print 'Run as user             : ', (getpwuid($>))[0]    ,"\n";
 print 'Start time              : ', scalar localtime $^T ,"\n";
-print "Module auth dir scripts : $module_dir\n";
-print "Authorization method    : ", $plg->auth_method ,"\n";
-print "Session directory       : ", $plg->dir_session ,"\n";
-print "Session idle timeout    : ", $plg->Session_timeout ," sec\n";
 print "Main PID                : $$\n";
+print 'Run as user             : ', (getpwuid($>))[0] ,"\n";
+print 'Authorization method    : ', $plg->auth_method ,"\n";
+print 'Session directory       : ', $plg->dir_session ,"\n";
+print 'Session idle timeout    : ', $plg->Session_timeout ," sec\n";
+print "Module auth dir scripts : $module_dir\n";
 print "version Perl            : $^V\n";
 print "version Dancer2         : $Dancer2::VERSION\n";
 print "version WebService      : $VERSION\n";
 
-
-# Restore the valid sessions, and delete the expired
+# Restore the valid sessions, and delete the expired ones
 opendir DIR, $plg->dir_session or die "Could not list session directory $plg->{dir_session} because $!\n";
 
 	foreach my $token (grep ! /^\.+$/, readdir DIR) {
@@ -173,37 +173,31 @@ opendir DIR, $plg->dir_session or die "Could not list session directory $plg->{d
 	}
 
 closedir DIR;
-#print 'debug : '. Dumper( $app )                   ;exit;
-#print 'debug : '. Dumper( $plg->config->{Routes} ) ;exit;
-#print 'debug : '. Dumper( $plg->auth_config )      ;exit;
-#print 'debug : '. Dumper  \%TokenDB; exit;
+#print Dumper( $app )                   ;exit;
+#print Dumper( $plg->config->{Routes} ) ;exit;
+#print Dumper( $plg->auth_config )      ;exit;
+#print Dumper  \%TokenDB; exit;
+
 
 # Hook, BEFORE the main app process the request
 
 	$app->add_hook(Dancer2::Core::Hook->new(name=>'before_request', code=>sub{
 	$plg->error(0);
-	$plg->data({});
-
-	# Route name
-	if    ( $app->request->{route}->{regexp} =~/^\^[\/\\]+(.*?)[\/\\]+\(\?#token.*/ )	{ $plg->route_name($1) }
-	elsif ( $app->request->{route}->{regexp} =~/^\^[\/\\]+(.*?)\$/ )					{ $plg->route_name($1) }
-	else  { $plg->error("Could not recognize route $_"); $app->halt($plg->reply) }
-
-	# Define the client IP address, even if the web service is running from a reverse proxy
-	$plg->ClientIP( $app->request->env->{HTTP_X_REAL_IP} // $app->request->address // '127.0.0.1' );
-
-	# format input/output
+	$plg->sort(   exists $app->request->query_parameters->{sort}   ? $app->request->query_parameters->{sort}  =~/(?i)1|t|y/ ? 1:0:0);  # sort   default is 0
+    $plg->pretty( exists $app->request->query_parameters->{pretty} ? $app->request->query_parameters->{pretty}=~/(?i)1|t|y/ ? 1:0:1);  # pretty default is 1
+	$plg->ClientIP($app->request->env->{HTTP_X_REAL_IP} // $app->request->address // '127.0.0.1'); # Client IP address, even if running from a reverse proxy
 
+		# format
 		foreach (qw/from to/) {
 
 			if (exists $app->request->query_parameters->{$_}) {
 
-				if ( $app->request->query_parameters->{$_} =~ $plg->formats_regex ) {
+				if ( $app->request->query_parameters->{$_} =~ $fmt_rgx ) {
 				$plg->Format->{$_} = $app->request->query_parameters->{$_}
 				}
 				else {
-				$plg->error("Format $_ ".$app->request->query_parameters->{$_}.' is not one of the supported : '. join(', ',keys %{$plg->formats}));
 				$plg->Format->{to}='json';
+				$plg->error("Format $_ ".$app->request->query_parameters->{$_}.' is not one of the supported : '. join(', ',keys %Formats));
 				$app->halt($plg->reply)
 				}
 			}
@@ -212,32 +206,25 @@ closedir DIR;
 			}
 		}
 
-	# if the output should be sorted, sort([0|1])
-	$plg->{sort} = ((exists $app->request->query_parameters->{sort}) && ($app->request->query_parameters->{sort} =~/(?i)1|t|y/)) ? 1:0;
-
-	# if the output should be human readable : pretty([0|1])
-	if (exists $app->request->query_parameters->{pretty}) {
-	$app->request->query_parameters->{pretty} =~/(?i)1|t|y/ ? $plg->pretty(1) : $plg->pretty(0)
-	}
-	else {
-	$plg->pretty(1)
-	}
-
 	# add header
-	$app->request->header('Content-Type'=> $plg->formats->{$plg->Format->{to}});
+	$app->request->header('Content-Type'=> $Formats{$plg->Format->{to}});
+
+	# route name
+	if    ( $app->request->{route}->{regexp} =~/^\^[\/\\]+(.*?)[\/\\]+\(\?#token.*/ )	{ $plg->route_name($1) }
+	elsif ( $app->request->{route}->{regexp} =~/^\^[\/\\]+(.*?)\$/ )					{ $plg->route_name($1) }
+	else  { $app->halt($plg->reply('error'=>'Could not recognize the route')) }
 
 	# Convert the posted data string, to hash $plg->data
+	$plg->data({});
 
-		if ($app->request->body) { 
+		if ($app->request->body) {
 
 			eval  {
 			if    ($plg->Format->{from} eq 'json')	{ $plg->data(Cpanel::JSON::XS::decode_json  Encode::encode('UTF-8',$app->request->body)) }
 			elsif ($plg->Format->{from} eq 'xml')	{ $plg->data(XML::Hash::XS::xml2hash $app->request->body) }
 			elsif ($plg->Format->{from} eq 'yaml')	{ $plg->data(YAML::Syck::Load        $app->request->body) }
 			elsif ($plg->Format->{from} eq 'perl')	{ $plg->data(eval                    $app->request->body) }
-			elsif ($plg->Format->{from} eq 'human')	{
-
-				my $arrayref;
+			elsif ($plg->Format->{from} eq 'human')	{ my $arrayref;
 
 					while ( $app->request->body =~/(.*)$/gm ) {
 					my @array = split /\s*(?:\,| |\t|-->|==>|=>|->|=|;|\|)+\s*/, $1;
@@ -257,19 +244,19 @@ closedir DIR;
 
 			if ($@) {
 			$@ =~s/[\s\v\h]+/ /g;
-			$plg->error('Data parsing as '.$plg->Format->{from}." failed because : $@");
-			$app->halt($plg->reply)
+			$app->halt($plg->reply('error'=>'Data parsing as '.$plg->Format->{from}." failed because $@"))
 			}
 		}
 	}));
 
 
+
+
 # Hook ONLY for the protected routes, before the main app do anything
+# halt if the session is expired, otherelse update the lastaccess
 
 	$app->add_hook(Dancer2::Core::Hook->new(name=>'before', code=>sub{
-	return if (! exists $plg->config->{Routes}->{$plg->route_name}) || (! $plg->config->{Routes}->{$plg->route_name}->{Protected});
-
-	# Halt if the session is expired, otherelse update the lastaccess
+	return unless (exists $plg->config->{Routes}->{$plg->route_name}) && $plg->config->{Routes}->{$plg->route_name}->{Protected};
 
 		if (exists $plg->data->{token}) {
 
@@ -294,7 +281,7 @@ closedir DIR;
 			}
 		}
 		else {
-		$app->halt($plg->reply('error' => "You must provide a token for using the protected route $plg->{route_name}"))
+		$app->halt($plg->reply('error' => "You must provide a token to use the protected route $plg->{route_name}"))
 		}
 
 	# Check if the user is member to all the route's Groups
@@ -307,6 +294,9 @@ closedir DIR;
 		}
 	}));
 
+
+
+
 	# Built in route /WebService list the routes
 	$app->add_route(
 	regexp => '/WebService',
@@ -316,13 +306,11 @@ closedir DIR;
 		delete @{$Routes}{qw/WebService login logout/};
 
 			$plg->reply(
-			Application	=> $_[0]->{name},
-			Routes		=> {
-				'Informational'	=> [ map {"WebService/$_"} qw(version client about) ],
-				'WebService'	=> [ map {"/$_"} qw(login logout) ],
+			Routes=>{
+				'Built in'		=> [ map {$_} qw(WebService WebService/version WebService/client WebService/about login logout) ],
 				'Application'	=> {
-					'Protected' => [ map {"/$_"} grep   $Routes->{$_}->{Protected}, sort keys %{$Routes} ],
-					'Public'    => [ map {"/$_"} grep ! $Routes->{$_}->{Protected}, sort keys %{$Routes} ]
+					'Protected' => [ map {$_} grep   $Routes->{$_}->{Protected}, sort keys %{$Routes} ],
+					'Public'    => [ map {$_} grep ! $Routes->{$_}->{Protected}, sort keys %{$Routes} ]
 					}
 				}
 			)
@@ -401,7 +389,6 @@ closedir DIR;
 		for (my $i=0; $i<@{$plg->rules_compiled}; $i++) {
 
 			if ( $plg->ClientIP =~ $plg->rules_compiled->[$i] ) {
-			#print '*Rule '.$plg->rules->[$i].' matched client IP '.$plg->ClientIP."*\n";
 			$plg->error(0);
 			last
 			}
@@ -545,11 +532,8 @@ $plg->reply_text('');
 
 	if ($@) {
 	$@=~s/[\v\h]+/ /g;
-	$plg->error("hash to string convertion failed because : $@");
-	''
-	}
-	else {
-	$plg->reply_text
+	$plg->error("hash to string convertion failed because $@");
+	$plg->reply_text('')
 	}
 }
 
@@ -558,9 +542,9 @@ $plg->reply_text('');
 #	Returns a reply as: json, xml, yaml, perl or human
 #	It always include the error
 #
-#	reply							error
-#	reply(   k1=>'v1', ... )		specific keys , values
-#	reply( { k1=>'v1', ... } )		specific keys , values
+#	reply						only the error
+#	reply(   k1=>'v1', ... )	specific keys , values
+#	reply( { k1=>'v1', ... } )	specific keys , values
 #
 sub reply :PluginKeyword
 {
@@ -578,13 +562,11 @@ my $plg = shift;
 			}
 		}
 		else {
-		# This the normal operation		
-		$plg->__HASH_TO_STRING({error=> $plg->error, @_} )
+		$plg->__HASH_TO_STRING({error=> $plg->error, @_} ) # This the normal
 		}
 	}
 	else {
-	# if no argument passed then we return only the error
-	$plg->__HASH_TO_STRING({error=> $plg->error})
+	$plg->__HASH_TO_STRING({error=> $plg->error}) # if no argument return only the error
 	}
 
 	if ($plg->error) {
@@ -609,9 +591,7 @@ my $plg=shift;
 	if (@_) {
 
 		if ('HASH' eq ref $plg->data) {
-		my %hash;
-		@hash{@_} = 1;
-		map { $_ , $plg->data->{$_} } grep exists $hash{$_}, keys %{$plg->data}
+		%{$plg->data}{grep exists $plg->data->{$_}, @_}
 		}
 		elsif ('ARRAY' eq ref $plg->data) {
 		my %hash;
@@ -765,7 +745,7 @@ Dancer2::Plugin::WebService - RESTful Web Services with login, persistent data,
 
 =head1 VERSION
 
-version 4.3.7
+version 4.3.8
 
 =head2 SYNOPSIS
 
